var View, addDomEvent, addId, addIdPartial, addNameToData, attrEscape, currentDepth, dataValue, extend, extractPlaceholder, htmlEscape, htmlParser, htmlUnescape, modelPath, modelText, parse, parseAttr, parseElement, parsePlaceholder, parsePlaceholderContent, parseString, reduceStack, renderer, startsEndBlock, textEvents, trimInner, unaliasName, _ref;
htmlParser = require('./htmlParser');
_ref = require('./parser'), modelPath = _ref.modelPath, parsePlaceholder = _ref.parsePlaceholder, parseElement = _ref.parseElement, parseAttr = _ref.parseAttr, addDomEvent = _ref.addDomEvent, textEvents = _ref.textEvents;
View = module.exports = function() {
  var self;
  self = this;
  this._views = {};
  this._paths = {};
  this._onBinds = {};
  this._depths = {};
  this._aliases = {};
  this._partialIds = {};
  this._templates = {};
  this._inline = '';
  this._idCount = 0;
  this._uniqueId = function() {
    return '$' + (self._idCount++).toString(36);
  };
};
View.prototype = {
  _find: function(name) {
    return this._views[name] || (function() {
      throw "Can't find view: " + name;
    })();
  },
  get: function(viewName, ctx, parentCtx, path, triggerPath) {
    var i, indicies, item, obj, out, parentView, paths, re, start, type, view, _len, _ref2;
    type = viewName.charAt(0);
    if (!(view = this._views[viewName])) {
      if (~(i = viewName.indexOf('$p'))) {
        start = type === '#' || type === '^' ? 1 : 0;
        parentView = viewName.substr(start, i - start);
        this._find(parentView);
        this.get(parentView, {
          $triggerPath: triggerPath
        });
        return this.get(viewName, ctx, parentCtx, null, triggerPath);
      }
      return '';
    }
    paths = parentCtx && parentCtx.$paths;
    if (path) {
      paths = this._paths[viewName] = paths ? [path].concat(paths) : [path];
    } else {
      this._paths[viewName] = paths || (paths = this._paths[viewName]);
    }
    if (ctx && ctx.splice && ctx.slice) {
      if (paths) {
        paths[0] += '.$#';
        parentCtx = Object.create(parentCtx);
        parentCtx.$paths = paths;
      }
      if (ctx.length) {
        if (type === '^') {
          return '';
        }
        out = '';
        indicies = parentCtx.$i || [];
        for (i = 0, _len = ctx.length; i < _len; i++) {
          item = ctx[i];
          obj = extend(parentCtx, item);
          obj.$i = [i].concat(indicies);
          out += view(obj);
        }
        return out;
      } else {
        if (type !== '^') {
          return '';
        }
      }
    } else {
      if ((type === '^' && ctx) || (type === '#' && !ctx)) {
        return '';
      }
    }
    ctx = extend(parentCtx, ctx);
    if ((ctx.$paths = paths) && (triggerPath || (triggerPath = ctx.$triggerPath))) {
      re = RegExp(paths[0].replace(/\$#|\./g, function(match) {
        if (match === '.') {
          return '\\.';
        } else {
          return '(\\d+)';
        }
      }));
      ctx.$i = (_ref2 = re.exec(triggerPath)) != null ? _ref2.slice(1).reverse() : void 0;
    }
    return view(ctx);
  },
  make: function(name, template, data) {
    var render, self;
    if (data == null) {
      data = {};
    }
    if (!data.$isString) {
      this.make(name + '$s', template, extend(data, {
        $isString: true
      }));
    }
    self = this;
    render = function(ctx) {
      render = parse(self, name, template, data, self._onBinds[name]);
      return render(ctx);
    };
    return this._views[name] = function(ctx) {
      return render(extend(data, ctx));
    };
  },
  before: function(name, before) {
    var fn;
    fn = this._find(name);
    return this._views[name] = function(ctx) {
      before(ctx);
      return fn(ctx);
    };
  },
  after: function(name, after) {
    var fn;
    fn = this._find(name);
    return this._views[name] = function(ctx) {
      setTimeout(after, 0, ctx);
      return fn(ctx);
    };
  },
  inline: function() {}
};
extend = function(parent, obj) {
  var key, out;
  if (typeof parent !== 'object') {
    if (typeof obj === 'object') {
      return obj;
    } else {
      return {};
    }
  }
  out = Object.create(parent);
  if (!obj) {
    return out;
  }
  for (key in obj) {
    out[key] = obj[key];
  }
  return out;
};
addId = function(attrs, uniqueId) {
  if (attrs.id == null) {
    return attrs.id = function() {
      return attrs._id = uniqueId();
    };
  }
};
addIdPartial = function(attrs, uniqueId, partialIds, partial) {
  var fn;
  if (attrs.id != null) {
    return;
  }
  fn = function() {
    fn = function() {
      return partialIds[partial] = attrs._id = uniqueId();
    };
    return attrs._id = partialIds[partial] || attrs.id();
  };
  return attrs.id = function() {
    return fn();
  };
};
View.htmlEscape = htmlEscape = function(s) {
  if (s == null) {
    return '';
  } else {
    return s.toString().replace(/&(?!\s)|</g, function(s) {
      if (s === '&') {
        return '&amp;';
      } else {
        return '&lt;';
      }
    });
  }
};
View.attrEscape = attrEscape = function(s) {
  if (s == null || s === '') {
    return '""';
  }
  s = s.toString().replace(/&(?!\s)|"/g, function(s) {
    if (s === '&') {
      return '&amp;';
    } else {
      return '&quot;';
    }
  });
  if (/[ =<>']/.test(s)) {
    return '"' + s + '"';
  } else {
    return s;
  }
};
View.trim = function(s) {
  if (s) {
    return htmlParser.uncomment(s).replace(/(?:^|\n)\s*/g, '');
  } else {
    return '';
  }
};
trimInner = function(s) {
  if (s) {
    return s.replace(/\n\s*/g, '');
  } else {
    return '';
  }
};
extractPlaceholder = function(text) {
  var content, match;
  match = /^([^\{\(]*)(\{{2,3}|\({2,3})([^\}\)]+)(?:\}{2,3}|\){2,3})([\s\S]*)/.exec(text);
  if (!match) {
    return;
  }
  content = /^\s*([\#^\/]?)\s*([^\s>]*)(?:\s+:([^\s>]+))?(?:\s*>\s*([^\s]+)\s*)?/.exec(match[3]);
  return {
    pre: trimInner(match[1]),
    escaped: match[2].length === 2,
    literal: match[2].charAt(0) === '{',
    type: content[1],
    name: content[2],
    alias: content[3],
    partial: content[4],
    post: trimInner(match[4])
  };
};
startsEndBlock = function(s) {
  return /^(?:\{{2,3}|\({2,3})[\/^](?:\}{2,3}|\){2,3})/.test(s);
};
currentDepth = function(data, queues) {
  return data.$depth + queues.length;
};
unaliasName = function(data, queues, name) {
  var aliasName, i, offset, remainder;
  if (name.charAt(0) !== ':') {
    return name;
  }
  i = name.indexOf('.');
  aliasName = name.substring(1, i);
  remainder = name.substr(i + 1);
  offset = currentDepth(data, queues) - data.$aliases[aliasName];
  if (offset !== offset) {
    throw "Can't find alias for " + name;
  }
  return Array(offset + 1).join('.') + remainder;
};
addNameToData = function(data, name) {
  if (name && !(name in data)) {
    return data[name] = {
      model: name
    };
  }
};
dataValue = function(data, name, model) {
  var path, value;
  if (path = modelPath(data, name)) {
    if ((value = model.get(path)) != null) {
      return value;
    } else {
      return model[path];
    }
  } else {
    return data[name];
  }
};
modelText = function(view, name, escape) {
  return function(data, model) {
    var text;
    text = dataValue(data, name, model);
    text = text != null ? text.toString() : '';
    if (escape) {
      text = escape(text);
    }
    return text;
  };
};
reduceStack = function(stack) {
  var attrs, bool, html, i, item, key, pushValue, value, _i, _len;
  html = [''];
  i = 0;
  for (_i = 0, _len = stack.length; _i < _len; _i++) {
    item = stack[_i];
    pushValue = function(value, isAttr) {
      if (value && value.call) {
        return i = html.push(value, '') - 1;
      } else {
        return html[i] += isAttr ? attrEscape(value) : value;
      }
    };
    switch (item[0]) {
      case 'start':
        html[i] += '<' + item[1];
        attrs = item[2];
        if ('id' in attrs) {
          html[i] += ' id=';
          pushValue(attrs.id, true);
        }
        for (key in attrs) {
          value = attrs[key];
          if (key === 'id') {
            continue;
          }
          if (value != null) {
            if (bool = value.bool) {
              pushValue(bool);
              continue;
            }
            html[i] += ' ' + key + '=';
            pushValue(value, true);
          } else {
            html[i] += ' ' + key;
          }
        }
        html[i] += '>';
        break;
      case 'chars':
        pushValue(item[1]);
        break;
      case 'end':
        html[i] += '</' + item[1] + '>';
    }
  }
  return html;
};
renderer = function(view, items, events) {
  return function(data) {
    var domEvents, event, html, item, model, modelEvents, _i, _len;
    model = view.model;
    modelEvents = model.__events;
    domEvents = view.dom.events;
    html = ((function() {
      var _i, _len, _results;
      _results = [];
      for (_i = 0, _len = items.length; _i < _len; _i++) {
        item = items[_i];
        _results.push((item.call ? item(data, model) : item));
      }
      return _results;
    })()).join('');
    for (_i = 0, _len = events.length; _i < _len; _i++) {
      event = events[_i];
      event(data, modelEvents, domEvents);
    }
    return html;
  };
};
parsePlaceholderContent = function(view, data, partialName, queues, popped, stack, events, block, match, isString, onBind, callbacks) {
  var alias, autoClosed, datum, endBlock, lastAutoClosed, lastPartial, literal, name, partial, partialText, startBlock, type, _ref2;
  literal = match.literal, type = match.type, name = match.name, alias = match.alias, partial = match.partial;
  if (partial && isString) {
    partial += '$s';
  }
  if (alias) {
    data.$aliases[alias] = currentDepth(data, queues);
  }
  name = unaliasName(data, queues, name);
  addNameToData(data, name);
  if (block && ((endBlock = type === '/') || (autoClosed = type === '^' && (!name || name === block.name) && block.type === '#'))) {
    name = block.name, partial = block.partial, literal = block.literal, lastPartial = block.lastPartial, lastAutoClosed = block.lastAutoClosed;
    popped.push(queues.pop());
    _ref2 = queues[queues.length - 1], stack = _ref2.stack, events = _ref2.events, block = _ref2.block;
    callbacks.onStartBlock(stack, events, block);
  }
  if (startBlock = type === '#' || type === '^') {
    lastPartial = partial;
    partial = type + partialName();
    block = {
      type: type,
      name: name,
      partial: partial,
      literal: literal,
      lastPartial: lastPartial,
      lastAutoClosed: autoClosed
    };
  }
  if (partial) {
    partialText = function(data, model) {
      view._onBinds[partial] = onBind;
      view._depths[partial] = currentDepth(data, queues);
      view._aliases[partial] = data.$aliases;
      return view.get(partial, dataValue(data, name || '.', model), data, modelPath(data, name, true));
    };
  }
  if ((datum = data[name]) != null) {
    if (datum.model && !startBlock) {
      if (!literal) {
        callbacks.onBind(name, partial, endBlock, lastAutoClosed, lastPartial);
      }
      callbacks.onModelText(partialText, name, endBlock);
    } else {
      callbacks.onText(partialText, datum, endBlock);
    }
  } else {
    callbacks.onText(partialText, '', endBlock);
  }
  if (startBlock) {
    queues.push({
      stack: stack = [],
      events: events = [],
      viewName: partial,
      block: block
    });
    return callbacks.onStartBlock(stack, events, block);
  }
};
parse = function(view, viewName, template, data, onBind) {
  var block, chars, events, partialCount, partialName, popped, queue, queues, stack, uniqueId, _fn, _i, _len;
  partialCount = 0;
  partialName = function() {
    return viewName + '$p' + partialCount++;
  };
  data.$depth = view._depths[viewName] || 0;
  data.$aliases = view._aliases[viewName] || {};
  if (data.$isString) {
    if (viewName === 'Title$s') {
      onBind = function(events, name) {
        return events.push(function(data, modelEvents) {
          var path;
          if (!(path = modelPath(data, name))) {
            return;
          }
          return modelEvents.bind(path, ['$doc', 'prop', 'title', 'Title$s']);
        });
      };
    }
    return parseString(view, viewName, template, data, partialName, onBind || function() {});
  }
  uniqueId = view._uniqueId;
  queues = [
    {
      stack: stack = [],
      events: events = []
    }
  ];
  popped = [];
  block = null;
  htmlParser.parse(template, {
    start: function(tag, tagName, attrs) {
      var attr, forAttr, out, parser, value;
      if (parser = parseElement[tagName]) {
        out = parser(events, attrs) || {};
        if (out.addId) {
          addId(attrs, uniqueId);
        }
      }
      forAttr = function(attr, value) {
        var anyOut, anyParser, bool, del, elOut, elParser, invert, literal, match, method, name, partial, post, pre, render, _ref2;
        if (match = extractPlaceholder(value)) {
          pre = match.pre, post = match.post, name = match.name, partial = match.partial, literal = match.literal;
          name = unaliasName(data, queues, name);
          addNameToData(data, name);
          invert = /^\s*!\s*$/.test(pre);
          if ((pre && !invert) || post || partial) {
            partial = partialName();
            addIdPartial(attrs, uniqueId, view._partialIds, partial);
            render = parseString(view, partial, value, data, partialName, function(events, name) {
              return events.push(function(data, modelEvents) {
                var path;
                if (!(path = modelPath(data, name))) {
                  return;
                }
                return modelEvents.bind(path, [attrs._id || attrs.id, 'attr', attr, partial]);
              });
            });
            view._views[partial] = function(ctx) {
              return render(extend(data, ctx));
            };
            attrs[attr] = function(data, model) {
              return attrEscape(render(data, model));
            };
            return;
          }
          if (parser = parsePlaceholder[attr]) {
            if (anyParser = parser['*']) {
              anyOut = anyParser(events, attrs, name, invert);
            }
            if (elParser = parser[tagName]) {
              elOut = elParser(events, attrs, name, invert);
            }
          }
          anyOut || (anyOut = {});
          elOut || (elOut = {});
          method = elOut.method || anyOut.method || 'attr';
          bool = elOut.bool || anyOut.bool;
          del = elOut.del || anyOut.del;
          if (((_ref2 = data[name]) != null ? _ref2.model : void 0) && !literal) {
            addId(attrs, uniqueId);
            events.push(function(data, modelEvents) {
              var args, path;
              if (!(path = modelPath(data, name))) {
                return;
              }
              args = [attrs._id || attrs.id, method, attr];
              if (invert) {
                args[3] = '$inv';
              }
              return modelEvents.bind(path, args);
            });
          }
          if (del) {
            delete attrs[attr];
          } else {
            attrs[attr] = bool ? {
              bool: function(data, model) {
                if (!dataValue(data, name, model) !== !invert) {
                  return ' ' + attr;
                } else {
                  return '';
                }
              }
            } : modelText(view, name, attrEscape);
          }
        }
        if (!(parser = parseAttr[attr])) {
          return;
        }
        if (anyParser = parser['*']) {
          anyOut = anyParser(events, attrs, value);
        }
        if (elParser = parser[tagName]) {
          elOut = elParser(events, attrs, value);
        }
        anyOut || (anyOut = {});
        elOut || (elOut = {});
        if (elOut.addId || anyOut.addId) {
          return addId(attrs, uniqueId);
        }
      };
      for (attr in attrs) {
        value = attrs[attr];
        if (attr === 'style') {
          continue;
        }
        forAttr(attr, value);
      }
      if ('style' in attrs) {
        forAttr('style', attrs.style);
      }
      return stack.push(['start', tagName, attrs]);
    },
    chars: chars = function(text) {
      var escaped, match, post, pre, pushText, wrap;
      if (!(match = extractPlaceholder(text))) {
        if (text = trimInner(text)) {
          stack.push(['chars', text]);
        }
        return;
      }
      pre = match.pre, post = match.post, escaped = match.escaped;
      pushText = function(text, endBlock) {
        if (text && !endBlock) {
          return stack.push(['chars', text]);
        }
      };
      pushText(pre);
      wrap = null;
      parsePlaceholderContent(view, data, partialName, queues, popped, stack, events, block, match, false, null, {
        onBind: function(name, partial, endBlock, lastAutoClosed, lastPartial) {
          var addEvent, attrs, i, last;
          i = stack.length - (endBlock ? (lastAutoClosed ? 3 : 2) : 1);
          last = stack[i];
          if (wrap = pre || (post && !startsEndBlock(post)) || !(last && last[0] === 'start')) {
            last = ['start', 'ins', {}];
            if (endBlock) {
              stack.splice(i + 1, 0, last);
            } else {
              stack.push(last);
            }
          }
          attrs = last[2];
          addId(attrs, uniqueId);
          if ('contenteditable' in attrs) {
            addDomEvent(events, attrs, name, textEvents, 'html');
          }
          addEvent = function(partial, domMethod) {
            return events.push(function(data, modelEvents) {
              var params, path;
              if (!(path = modelPath(data, name))) {
                return;
              }
              params = [attrs._id || attrs.id, domMethod, +escaped];
              if (partial) {
                params[3] = partial;
              }
              return modelEvents.bind(path, params);
            });
          };
          addEvent(partial, 'html');
          if (lastAutoClosed) {
            return addEvent(lastPartial, 'appendHtml');
          }
        },
        onModelText: function(partialText, name, endBlock) {
          if (partialText) {
            escaped = false;
          }
          pushText(partialText || modelText(view, name, escaped && htmlEscape), endBlock);
          if (wrap) {
            return stack.push(['end', 'ins']);
          }
        },
        onText: function(partialText, value, endBlock) {
          if (partialText) {
            escaped = false;
          }
          return pushText(partialText || (escaped ? htmlEscape(value) : value.toString()), endBlock);
        },
        onStartBlock: function(_stack, _events, _block) {
          stack = _stack;
          events = _events;
          return block = _block;
        }
      });
      if (post) {
        return chars(post);
      }
    },
    end: function(tag, tagName) {
      return stack.push(['end', tagName]);
    }
  });
  _fn = function(queue) {
    var render;
    render = renderer(view, reduceStack(queue.stack), queue.events);
    return view._views[queue.viewName] = function(ctx) {
      return render(extend(data, ctx));
    };
  };
  for (_i = 0, _len = popped.length; _i < _len; _i++) {
    queue = popped[_i];
    _fn(queue);
  }
  return renderer(view, reduceStack(stack), events);
};
htmlUnescape = function(s) {
  return s.replace('&rpar;', ')').replace('&lpar;', '(');
};
parseString = function(view, viewName, template, data, partialName, onBind) {
  var block, events, match, popped, post, pre, pushText, queue, queues, stack, _fn, _i, _len;
  queues = [
    {
      stack: stack = [],
      events: events = []
    }
  ];
  popped = [];
  block = null;
  pushText = function(text, endBlock) {
    if (text && !endBlock) {
      return stack.push(text);
    }
  };
  post = template;
  while (post) {
    match = extractPlaceholder(post);
    if (match == null) {
      pushText(htmlUnescape(post));
      break;
    }
    pre = match.pre, post = match.post;
    pushText(htmlUnescape(pre));
    parsePlaceholderContent(view, data, partialName, queues, popped, stack, events, block, match, true, onBind, {
      onBind: function(name) {
        return onBind(events, name);
      },
      onModelText: function(partialText, name, endBlock) {
        return pushText(partialText || modelText(view, name), endBlock);
      },
      onText: function(partialText, value, endBlock) {
        return pushText(partialText || value.toString(), endBlock);
      },
      onStartBlock: function(_stack, _events, _block) {
        stack = _stack;
        events = _events;
        return block = _block;
      }
    });
  }
  _fn = function(queue) {
    var render;
    render = renderer(view, queue.stack, queue.events);
    return view._views[queue.viewName] = function(ctx) {
      return render(extend(data, ctx));
    };
  };
  for (_i = 0, _len = popped.length; _i < _len; _i++) {
    queue = popped[_i];
    _fn(queue);
  }
  return renderer(view, stack, events);
};