var Dom, View, empty, emptyModel, emptyRes, loader, modelHelper, uglify;
uglify = require('uglify-js');
module.exports = View = require('./View');
Dom = require('./Dom');
modelHelper = require('./modelHelper');
loader = require('./loader');
empty = function() {};
emptyRes = {
  getHeader: empty,
  setHeader: empty,
  write: empty,
  end: empty
};
emptyModel = {
  get: empty,
  bundle: empty
};
View.prototype.before = View.prototype.after = function() {};
View.prototype.inline = function(fn) {
  return this._inline += uglify("(" + fn + ")()") + ';';
};
View.prototype._load = function(callback) {
  var self;
  if (loader.isProduction) {
    this._load = function(callback) {
      return callback();
    };
  }
  self = this;
  return loader.js(this, this._appFilename, this._derbyOptions, function(obj) {
    self._root = obj.root;
    self._clientName = obj.clientName;
    self._jsFile = obj.jsFile;
    self._require = obj.require;
    return callback();
  });
};
View.prototype._init = function(model) {
  this.dom = new Dom(this.model = modelHelper.init(model));
  return this._idCount = 0;
};
View.prototype.send = function(res, model, ctx) {
  var dom, self;
  if (res == null) {
    res = emptyRes;
  }
  if (model == null) {
    model = emptyModel;
  }
  self = this;
  this._init(model);
  dom = this.dom;
  return this._load(function() {
    return loader.css(self._root, self._clientName, function(css) {
      return self._send(res, model, ctx, dom, css);
    });
  });
};
View.prototype._send = function(res, model, ctx, dom, css) {
  var clientName, doctype, head, header, initEnd, initStart, tail, title;
  if (!res.getHeader('content-type')) {
    res.setHeader('Content-Type', 'text/html; charset=utf-8');
  }
  doctype = this.get('Doctype', ctx) || '<!DOCTYPE html><meta charset=utf-8>';
  title = View.htmlEscape(this.get('Title$s', ctx)) || 'Derby app';
  head = this.get('Head', ctx);
  header = this.get('Header', ctx);
  res.write("" + doctype + "<title>" + title + "</title>" + head + "<style>" + css + "</style>" + header);
  res.write(this.get('Body', ctx));
  clientName = this._clientName;
  res.write("<script>function $(i){return document.getElementById(i)}" + ("function " + clientName + "(){" + clientName + "=1}") + ("" + this._inline + "</script>") + this.get('Script', ctx) + ("<script defer async onload=" + clientName + "() src=" + this._jsFile + "></script>"));
  tail = this.get('Tail', ctx);
  initStart = "<script>(function(){function f(){setTimeout(function(){" + ("" + clientName + "=require('./" + this._require + "')(" + this._idCount + ",") + JSON.stringify(this._paths) + ',' + JSON.stringify(this._partialIds) + ',' + JSON.stringify(this._aliases) + ',' + JSON.stringify(this._depths) + ',';
  initEnd = ',' + JSON.stringify(model.__events.get()) + ',' + JSON.stringify(dom.events.get()) + (")},0)}" + clientName + "===1?f():" + clientName + "=f})()</script>" + tail);
  return model.bundle(function(bundle) {
    return res.end(initStart + bundle.replace(/<\//g, '<\\/') + initEnd);
  });
};