var basename, crypto, cssCache, dirname, exists, fs, isProduction, join, jsCache, nib, racer, stylus, trim, views, _ref;
_ref = require('path'), dirname = _ref.dirname, basename = _ref.basename, join = _ref.join, exists = _ref.exists;
fs = require('fs');
crypto = require('crypto');
stylus = require('stylus');
nib = require('nib');
racer = require('racer');
trim = require('./View').trim;
cssCache = {};
jsCache = {};
module.exports = {
  isProduction: isProduction = process.env.NODE_ENV === 'production',
  css: function(root, clientName, callback) {
    var css, path;
    path = join(root, 'styles', clientName, 'index.styl');
    if (isProduction && (css = cssCache[path])) {
      return callback(css);
    }
    if (!clientName) {
      return callback('');
    }
    return fs.readFile(path, 'utf8', function(err, styl) {
      if (err) {
        return callback(cssCache[path] = '');
      }
      return stylus(styl).use(nib()).set('filename', path).set('compress', true).render(function(err, css) {
        if (err) {
          throw err;
        }
        css = trim(css);
        callback(css);
        return cssCache[path] = css;
      });
    });
  },
  views: views = function(view, root, clientName, callback) {
    var dir;
    dir = join(root, 'views', clientName);
    return fs.readdir(dir, function(err, files) {
      var count, file, _i, _len, _results;
      if (!files) {
        return callback();
      }
      count = files.length;
      _results = [];
      for (_i = 0, _len = files.length; _i < _len; _i++) {
        file = files[_i];
        _results.push((function(file) {
          return fs.readFile(join(dir, file), 'utf8', function(err, template) {
            var scope, viewName, _ref2;
            _ref2 = file.split('.'), viewName = _ref2[0], scope = _ref2[1];
            view.make(viewName, template);
            if (scope !== 'server') {
              view._templates[viewName] = trim(template);
            }
            if (!--count) {
              return callback();
            }
          });
        })(file));
      }
      return _results;
    });
  },
  js: function(view, parentFilename, options, callback) {
    var base, bundle, clientName, dir, finish, js, minify, root, staticDir, staticPath, staticRoot;
    if (!parentFilename) {
      return callback({});
    }
    if ((base = basename(parentFilename, '.js')) === 'index') {
      base = basename(dirname(parentFilename));
      dir = dirname(parentFilename);
      root = dirname(dirname(dir));
    } else {
      root = dirname(parentFilename);
    }
    clientName = options.name || base;
    staticRoot = options.root || join(root, 'public');
    staticDir = options.dir || 'gen';
    staticPath = join(staticRoot, staticDir);
    finish = function(js) {
      return views(view, root, clientName, function() {
        var filePath, filename, jsFile;
        js = js.replace('"{{templates}}"', JSON.stringify(view._templates || {}));
        filename = crypto.createHash('md5').update(js).digest('base64') + '.js';
        filename = filename.replace(/[\/\+=]/g, function(match) {
          switch (match) {
            case '/':
              return '-';
            case '+':
              return '_';
            case '=':
              return '';
          }
        });
        jsFile = join('/', staticDir, filename);
        filePath = join(staticPath, filename);
        return fs.writeFile(filePath, js, function() {
          return callback({
            root: root,
            clientName: clientName,
            jsFile: jsFile,
            require: basename(parentFilename)
          });
        });
      });
    };
    minify = 'minify' in options ? options.minify : isProduction;
    bundle = isProduction || !(js = jsCache[parentFilename]) ? function() {
      return fs.readFile(join(dir, 'inline.js'), 'utf8', function(err, inline) {
        racer.js({
          minify: minify,
          require: parentFilename
        }, function(js) {
          return finish(jsCache[parentFilename] = js);
        });
        if (err) {
          return;
        }
        return view.inline("function(){" + inline + "}");
      });
    } : function() {
      return finish(js);
    };
    return exists(staticPath, function(value) {
      if (value) {
        return bundle();
      }
      return exists(staticRoot, function(value) {
        if (value) {
          return fs.mkdir(staticPath, 0777, function(err) {
            return bundle();
          });
        }
        return fs.mkdir(staticRoot, 0777, function(err) {
          return fs.mkdir(staticPath, 0777, function(err) {
            return bundle();
          });
        });
      });
    });
  }
};